/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.trifork.gravidcompose.theme

import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.material3.ColorScheme
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.staticCompositionLocalOf

@Composable
fun GravidThemePropertiesProvider(
    dimens: GravidDimens,
    typography: GravidTypography,
    content: @Composable () -> Unit,
) {
    CompositionLocalProvider(
        LocalGravidDimens provides dimens,
        LocalGravidTypography provides typography,
        content = content,
    )
}

private val LocalGravidDimens = staticCompositionLocalOf<GravidDimens> {
    error("No dimens found")
}
private val LocalGravidTypography = staticCompositionLocalOf<GravidTypography> {
    error("No typography found")
}

@Composable
fun GravidTheme(
    overrideColorScheme: ColorScheme? = null,
    darkTheme: Boolean = isSystemInDarkTheme(),
    content: @Composable () -> Unit,
) {
    val colorScheme = when {
        overrideColorScheme != null -> overrideColorScheme
        darkTheme -> DarkColorScheme
        else -> LightColorScheme
    }
    val dimens = GravidDimens()
    val typography = GravidTypography(
        colors = colorScheme,
        dimens = dimens,
        fontFamily = ptSansFamily,
    )

    MaterialTheme(colorScheme = colorScheme) {
        GravidThemePropertiesProvider(dimens = dimens, typography = typography) {
            CompositionLocalProvider(content = content)
        }
    }
}

object GravidTheme {
    val typography: GravidTypography
        @Composable
        get() = LocalGravidTypography.current
    val dimens: GravidDimens
        @Composable
        get() = LocalGravidDimens.current
}
